#include "mex.h"
#include <string.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (nrhs < 2)
		mexErrMsgIdAndTxt("MATLAB:minrhs", "Not enough input arguments.");
	if (nlhs > 1)
		mexErrMsgIdAndTxt("MATLAB:maxlhs", "Too many output arguments.");

	const mxArray *in = prhs[0];
	if (!mxIsDouble(in))
		mexErrMsgTxt("Unexpected argument type (not double).");
	mxComplexity ic = mxIsComplex(in);
	mxClassID it = mxDOUBLE_CLASS;

	const mxArray *size = 0;
	mwSize nd;
	if (nrhs == 2)
	{
		size = prhs[1];
		if (!mxIsNumeric(size) || mxGetNumberOfDimensions(size) != 2 || mxGetM(size) != 1 || mxIsSparse(size) || mxIsComplex(size))
			mexErrMsgTxt("Invalid size.");
		nd = mxGetN(size);
	}
	else
	{
		nd = nrhs - 1;
	}

	mwSize ind = mxGetNumberOfDimensions(in);
	mwSize *d = mxMalloc(sizeof(mwSize)*(nd + ind));
	memcpy(&d[nd], mxGetDimensions(in), sizeof(mwSize)*ind);

	mwSize dup = 1;
	if (size)
	{
		mwIndex i;
		#define GETDIMS(TYPE) { \
		TYPE *data = mxGetData(size); \
		for (i = 0; i < nd; i ++) \
			dup *= (d[i] = (mwSize)data[i]); \
		} break
		switch (mxGetClassID(size))
		{
			case mxDOUBLE_CLASS: GETDIMS(double);
			case mxSINGLE_CLASS: GETDIMS(float);
			case mxINT8_CLASS: GETDIMS(int8_T);
			case mxUINT8_CLASS: GETDIMS(uint8_T);
			case mxINT16_CLASS: GETDIMS(int16_T);
			case mxUINT16_CLASS: GETDIMS(uint16_T);
			case mxINT32_CLASS: GETDIMS(int32_T);
			case mxUINT32_CLASS: GETDIMS(uint32_T);
			case mxINT64_CLASS: GETDIMS(int64_T);
			case mxUINT64_CLASS: GETDIMS(uint64_T);
			default: mexErrMsgTxt("Invalid size type.");
		}
	}
	else
	{
		int i;
		for (i = 0; i < nd; i ++)
		{
			if (!mxIsNumeric(prhs[i+1]) || mxIsComplex(prhs[i+1]))
				mexErrMsgTxt("Invalid size.");
			dup *= (d[i] = (mwSize)mxGetScalar(prhs[i+1]));
		}
	}

	size_t len = mxGetNumberOfElements(in);
	mxArray *out = mxCreateNumericArray(nd + ind, d, it, ic);
	if (!out)
		mexErrMsgTxt("Out of memory.");

	const double *ip = mxGetPr(in);
	double *op = mxGetPr(out);
	mwIndex i, j;
	for (i = 0; i < len; i++, ip ++)
		for (j = 0; j < dup; j ++, op ++)
			*op = *ip;
	if (ic)
	{
		ip = mxGetPi(in);
		op = mxGetPi(out);
		for (i = 0; i < len; i++, ip ++)
			for (j = 0; j < dup; j ++, op ++)
				*op = *ip;
	}

	plhs[0] = out;
}
