function p = param_deriv(p, n, d)
%PARAM_DERIV/PARAM_DERIV	Parameter vector constructor for PDERIV
%   P = PARAM_DERIV(PV) returns a vector of parameters P than can be used in PDERIV operations.
%   P = PARAM_DERIV(PV, N) specifies you only want N levels of partial derivatives.
%       N=0: constant (no pderiv)
%       N=1: gradient (default)
%       N=2: hessian
%   P = PARAM_DERIV(PV, N, D) specifies that differentiable parameters lie along dimension(s) D, and are common across all other dimensions (default = 1:ndims(PV)).

if nargin < 2
    n = 1;
end
assert(n >= 0);

if isa(p, 'param_deriv')
    p = p.p;
end
assert(isnumeric(p));

if nargin < 3
    d = 1:ndims(p);
end
s = size(p);
s = prod(s(d));
assert(isscalar(s));

p = class(struct('p', p, 'n', uint8(n), 'd', d, 's', s), 'param_deriv');
