function d = subparam(d, i)
%PDERIV/SUBPARAM    Select a sub/superset of parameters to keep derivatives of
%   DI = SUBPARAM(D, I) creates a new derivative representation, only containing the parameters specified by the vector I.  A element of 0 in I creates a new parameter with a zero derivative.
%   Care must be exercized to make sure that DI is not combined with D, but only with other derivative representations with the same parameters.
%
%   For example, to temporarily operate only on parameters 3 and 5 out of 10, do:
%      p = param_deriv(randn(1,10));
%      i = [3 5];
%      x = subparam(p(i), i);
%   Now x only has two parameters.
%      y = ... x ...
%      j = zeros(1,10);
%      j(i) = 1:length(i);
%      y = subparam(y, j);
%   Now y has all ten parameters again.

assert(isvector(i) && all(i >= 0), 'Invalid parameter set');
i = i(:);

i0 = i == 0;
i1 = ~i0;
is = i(i1);

if isobject(d.d)
    d.d = subparam(d.d, i);
end
dis = pderiv_subs(d.d, {is, ':'});
d.d = pderiv_setsub(pderiv_setdepth(zeros([length(i) size(d.v)]), pderiv_depth1(d.d), length(i)), {i1, ':'}, dis);
