function z = times(x,y)
%PDERIV/TIMES	Multiply
%   Z = X .* Y computes the derivative representation of the product X*Y, computed as X*Y'+X'*Y for each element

if ~isobject(x)
    y.v = x .* y.v;
    y.d = times01(x, y.d);
    z = y;
elseif ~isobject(y)
    x.v = x.v .* y;
    x.d = times01(y, x.d);
    z = x;
else
    x0 = pderiv_d0(x);
    y0 = pderiv_d0(y);
    x.v = x.v .* y.v;
    x.d = times01(y0,x.d) + times01(x0,y.d);
    z = x;
end

% optimization of map01
function z = bsxtimes(x,y)
    if ~isobject(x)
        if ~isobject(y)
            z = bsxfun(@times, x, y);
        else
            y.v = bsxfun(@times, x, y.v);
            y.d = times01(x, y.d);
            z = y;
        end
    elseif ~isobject(y)
        x.v = bsxfun(@times, x.v, y);
        x.d = times01(y, x.d);
        z = x;
    else
        x0 = pderiv_d0(x);
        y0 = pderiv_d0(y);
        x.v = bsxfun(@times, x.v, y.v);
        x.d = times01(y0,x.d) + times01(x0,y.d);
        z = x;
    end

function d = times01(d0, d1)
    d = bsxtimes(shiftdim(d0, -1), d1);
