% PDERIV	Create and manipulate analytical derivative representations
% (c) 2007-2008 Dylan Simon <dylan@dylex.net>, NYU
%
%   param_deriv - Parameter vector constructor
%   pderiv      - Derivative representation constructor
%   pderiv_val  - Get final constant value result
%   pderiv_fun  - Transparently wrap an existing function to return derivatives
%
%
% Example:
%
%   You have some parameters:
%       p = [ alpha beta gamma ... ];
%   To indicate you want to start computing 1 level partial-derivatives with
%   respect to these parameters, you make them into a param_deriv object:
%       p = param_deriv(p, 1);
%   Each of these parameters will now only be referred to only by indexing this
%   param_deriv object.  Beyond that, you just do your calculations as usual.
%
%   Suppose you want a vector of n values, v, that are initialized to 0.  As
%   usual you say:
%       v = zeros(n,1);
%   Alternatively, you might want to initialize them with one of your
%   parameters, say p(5), then instead you say:
%       v = p(repmat(5,n,1));
%   Now, let's say you want to do an update of v using a vector of rewards r
%   and successor values nv with discount p(3) and rate p(1), so you just:
%       d = -v + r + p(3) .* nv;
%       v = v + p(1) .* d;
%
%   Your resulting values are v.d0 and your derivatives are dv/dp(n) = v.d1(n).
%   You can use v.d1 to get a matrix of derivatives with parameters in the
%   first dimension and the rest of the dimensions shifted (so a column vector
%   becomes the columns in the matrix).
%
%   A simpler (but more limited) way to use pderiv is with deriv_fun, which
%   converts a normal function to one which can return derivatives.
