function varargout = comm(varargin)
%COMM	Make arrays commensurate
%   [x y ...] = comm(x, y, ...) makes all arguments of the same size by replcating singular dimensions out to the common size, leaving scalars.

doscalar = 0;
n = nargin;

assert(nargout == n, 'comm: argument mismatch');
varargout = varargin;

s = cellfun(@size, varargin, 'UniformOutput', false);
sl = cellfun('length', s);
sm = max(sl);
s = cellfun(@(s)[s ones(1,sm-length(s)-1)], s, 'UniformOutput', false);
s = vertcat(s{:});

m = max(s);
if prod(m) == 1
    return
end
for i = 1:n
    if ~isscalar(varargout{i}) || doscalar
        si = s(i,:);
        s1 = si == 1;
        assert(all(s1 | (si == m)), 'comm: nonconformant arguments');

        varargout{i} = repmat(varargout{i}, 1+s1.*(m-1));
    end
end
