function fd = deriv_fun(f)
%DERIV_FUN  Convert a normal function to one that computes derivatives
%   DF = DERIV_FUN(@F) returns a function handle, DF, such that if:
%     v = F(X, ARGS...)
%   for some numeric vector X, then:
%     [v dv ddv ...] = DF(X, ARGS)
%   where
%     dv = dv/dX is the derivative(s) of v with respect to X,
%     ddv = d^2v/dX^2 is the 2nd, etc.
%   Derivatives are computed as needed, and if only one output argument is
%   requested, there is no additional computational cost.

    fd = @(p, varargin) wrap(f, p, varargin{:});

end

function [v varargout] = wrap(f, p, varargin)

    nd = nargout - 1;   
    p = param_deriv(p, nd);
    v = f(p(:), varargin{:});
    
    [v varargout{1:nd}] = derivs(v);

end
